#include <stdint.h>

#include <avr/io.h>

#include "ir.h"
#include "ir_sircs.h"

enum ir_sircs_receiving_stage_t ir_sircs_receiving_stage = IR_SIRCS_RECEIVING_START;

uint8_t incoming_command;
uint8_t incoming_address;
uint8_t bits_received;

uint8_t repeat_count;
uint8_t can_repeat;

// Call to indicate a signal edge from the IR receiver has been captured.
void ir_sircs_edge_captured(enum ir_edge_direction_t edge, uint16_t delta_time) {
	if (edge == IR_EDGE_RISING) {
		// Was the gap legal?
		if (delta_time < IR_SIRCS_DATA_BIT_MIN || delta_time > IR_SIRCS_DATA_BIT_MAX) {
			ir_sircs_receiving_stage = IR_SIRCS_RECEIVING_START;
		}
		// Can we clear the repeat?
		if (delta_time > IR_SIRCS_REPEAT_DELAY_MAX) {
			can_repeat = 0;
		}
	} else {
		// What's the type of bit?
		if (ir_sircs_receiving_stage == IR_SIRCS_RECEIVING_START && delta_time >= IR_SIRCS_START_BIT_MIN && delta_time <= IR_SIRCS_START_BIT_MAX) {
			// Start receiving the command.
			ir_sircs_receiving_stage = IR_SIRCS_RECEIVING_COMMAND;
			incoming_command = 0;
			bits_received = 0;
		} else if (ir_sircs_receiving_stage == IR_SIRCS_RECEIVING_COMMAND && delta_time >= IR_SIRCS_DATA_BIT_MIN && delta_time <= IR_SIRCS_DATA_BIT_MAX) {
			// Receive a bit of the command.
			if (delta_time >= IR_SIRCS_DATA_BIT_THRESHOLD) incoming_command |= _BV(bits_received);
			if (++bits_received == 7) {
				ir_sircs_receiving_stage = IR_SIRCS_RECEIVING_ADDRESS;
				incoming_address = 0;
				bits_received = 0;
			}
		 } else if (ir_sircs_receiving_stage == IR_SIRCS_RECEIVING_ADDRESS && delta_time >= IR_SIRCS_DATA_BIT_MIN && delta_time <= IR_SIRCS_DATA_BIT_MAX) {
			// Receive a bit of the address.
			if (delta_time >= IR_SIRCS_DATA_BIT_THRESHOLD) incoming_address |= _BV(bits_received);
			if (++bits_received == 5) {
				// We're done.
				ir_sircs_receiving_stage = IR_SIRCS_RECEIVING_START;
				if (can_repeat) {
					if (repeat_count != 0xFF) ++repeat_count;
				} else {
					repeat_count = 0;
					can_repeat = 1;
				}
				ir_sircs_code_received(incoming_address, incoming_command, repeat_count);
			}
		} else {
			// No idea what that was, wait for a start bit.
			ir_sircs_receiving_stage = IR_SIRCS_RECEIVING_START;
		}
	}
}